(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright (c) 2013-2014 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uhomogeneous_tuple_spec.mli,v 1.19 2014/06/11 15:45:34 fclement Exp $ *)

(** Homogeneous tuples specification. *)

type dimension = Unatural.t;;
type component_number = Unatural.t;;

module type STRUCTURE =
sig
  type 'a t;;
  
  val get_dim : unit -> dimension;;
  
  val print : ('a, 'a t) Format_printer.printer_lifter;;
  
  val get : 'a t -> component_number -> 'a;;
  val init : (component_number -> 'a) -> 'a t;;
  val of_list : 'a list -> 'a t;;
  val to_list : 'a t -> 'a list;;
  val of_triple : 'a * 'a * 'a -> 'a t;;
  val to_triple : 'a t -> 'a * 'a * 'a;;
  val for_all : ('a -> bool) -> 'a t -> bool;;
  val for_all2 : ('a -> 'b -> bool) -> 'a t -> 'b t -> bool;;
  val iter : ('a -> unit) -> 'a t -> unit;;
  val iter_i : (component_number -> 'a -> unit) -> 'a t -> unit;;
  val iter_rails : ('a -> unit) t -> 'a t -> unit;;
  val fold_left : ('accu -> 'a -> 'accu) -> 'accu -> 'a t -> 'accu;;
  val fold_left_i :
    ('accu -> component_number -> 'a -> 'accu) -> 'accu -> 'a t -> 'accu
  ;;
  val iter2 : ('a -> 'b -> unit) -> 'a t -> 'b t -> unit;;
  val fold_left2 :
    ('accu -> 'a -> 'b -> 'accu) -> 'accu -> 'a t -> 'b t -> 'accu
  ;;
  val iter3 : ('a -> 'b -> 'c -> unit) -> 'a t -> 'b t -> 'c t -> unit;;
  val iter4 :
    ('a -> 'b -> 'c -> 'd -> unit) -> 'a t -> 'b t -> 'c t -> 'd t -> unit
  ;;
end
;;

module type HOMOGENEOUS_TUPLE =
sig
  include STRUCTURE;;
  
  val make : 'a -> 'a t;;
  
  val same : ('a -> 'a -> bool) -> 'a t -> 'a t -> bool;;
  
  val singleton : component_number -> 'a -> 'a -> 'a t;;
  val of_array : 'a array -> 'a t;;
  val to_array : 'a t -> 'a array;;
  
  val map : ('a -> 'b) -> 'a t -> 'b t;;
  val map_i : (component_number -> 'a -> 'b) -> 'a t -> 'b t;;
  val map2 : ('a -> 'b -> 'c) -> 'a t -> 'b t -> 'c t;;
  val map3 : ('a -> 'b -> 'c -> 'd) -> 'a t -> 'b t -> 'c t -> 'd t;;
  val map4 :
    ('a -> 'b -> 'c -> 'd -> 'e) -> 'a t -> 'b t -> 'c t -> 'd t -> 'e t
  ;;
  val map_rails : ('a -> 'b) t -> 'a t -> 'b t;;
  val map_rails2 : ('a -> 'b -> 'c) t -> 'a t -> 'b t -> 'c t;;
  val iter_rails2 : ('a -> 'b -> unit) t -> 'a t -> 'b t -> unit;;
  val iter_rails3 :
    ('a -> 'b -> 'c -> unit) t -> 'a t -> 'b t -> 'c t -> unit
  ;;
  
  val components : ('a -> 'b t) -> ('a -> 'b) t;;
  val array_components : 'a t array -> 'a array t;;
  val components_map : ('a -> 'b) -> ('c -> 'a) t -> ('c -> 'b) t;;
  
  val split : ('a * 'b) t -> 'a t * 'b t;;
  
  val combine :  ('a -> 'b) t -> 'a -> 'b t;;
end

module type MAKER = functor (S : STRUCTURE) -> HOMOGENEOUS_TUPLE;;
