CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C               EPI Pomdapi, INRIA Paris - Rocquencourt               C
C                                                                     C
C  Copyright 2005-2014 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C $Id: write_memory.fpp,v 1.9 2014/03/13 20:08:47 fclement Exp $

#include "pio_constants.h"

      subroutine write_memory_location(sbr, io, loc)
C     
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, loc
      
C     Local variable.
      character byt
      
C     External function.
      character rand_fetch_char
      
      byt = rand_fetch_char(sbr, io, loc)
      
      write(PIO_REPORT, *) 'loc ', loc, ': |', byt, '|'
      
      return
      end
      
      
      subroutine write_communication_name(io)
C     Write the name of the communication.
      
      implicit none
      
C     Input argument.
      integer io
      
C     Local variables.
      character sbr*24, cname*PIO_MAX_IDENT_LENGTH
      integer tc, bn, en, len
      
C     External functions.
      integer fetch_communication_type
      integer fetch_communication_name_begin
      integer fetch_communication_name_end
      
      sbr = 'write_communication_name'
      
      tc = fetch_communication_type(sbr, io)
      
      if ((tc .eq. PIO_TASK) .or. (tc .eq. PIO_SERVICE)) then
         
         bn = fetch_communication_name_begin(sbr, io)
         en = fetch_communication_name_end(sbr, io)
         
         len = en - bn
         call rand_fetch_string(sbr, io, bn, cname, 1, len)
         
         write(PIO_REPORT, *) ' '
         write(PIO_REPORT, *)
     +        '[Pio/Fortran]',
     +        '  Communication_Name [', bn, '; ', en, '[ = ',
     +        '"', cname(1:len), '"'
         
      endif
      
      return
      end

      
      subroutine write_string_simple(io, i, k, tab)
C     Write a string simple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, k
      
C     Local variables.
      character sbr*19, sdatum*MAX_STRING_LENGTH
      integer len
      
      sbr = 'write_string_simple'
      
      len = MAX_STRING_LENGTH
      
      call rand_fetch_string_value(sbr, io, i, k, len, sdatum)
      
      write(PIO_REPORT, *) tab, '"', sdatum(1:len), '"'
      
      return
      end
      
      
      subroutine write_int_simple(io, i, j, k, sz, tab)
C     Write an integer simple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, k, sz
      
C     Local variables.
      character sbr*16
      integer idatum
      
C     External function.
      integer rand_fetch_int_value
      
      sbr = 'write_int_simple'
      
      idatum = rand_fetch_int_value(sbr, io, i, j, k, sz, 1, 1)
      
      write(PIO_REPORT, *) tab, idatum
      
      return
      end
      
      
      subroutine write_float_simple(io, i, j, k, sz, tab)
C     Write a float simple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, k, sz
      
C     Local variables.
      character sbr*18
      FLOAT fdatum
      
C     External function.
      FLOAT rand_fetch_float_value
      
      sbr = 'write_float_simple'
      
      fdatum = rand_fetch_float_value(sbr, io, i, j, k, sz, 1, 1)
      
      write(PIO_REPORT, 1001) tab, fdatum
      
 1001 format (1x, a, PRINT_FLOAT_FORMAT)
      
      return
      end
      
      
      subroutine write_num_simple(io, i, j, k, sz, typ, tab)
C     Write a couple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, k, sz, typ
      
C     External function.
      logical is_scalar_simple_int_type
      
      if (is_scalar_simple_int_type(typ)) then
         call write_int_simple(io, i, j, k, sz, tab)
      else
         call write_float_simple(io, i, j, k, sz, tab)
      endif
      
      return
      end
      
      
      subroutine write_simple(io, i, j, k, sz, typ, tab)
C     Write a  simple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, k, sz, typ
      
C     Local variable.
      character sbr*12
      
C     External functions.
      logical is_scalar_simple_string_type
      
      sbr = 'write_simple'
      
      if (is_scalar_simple_string_type(typ)) then
         call write_string_simple(io, i, k, tab)
      else
         call write_num_simple(io, i, j, k, sz, typ, tab)
      endif
      
      return
      end
      
      
      subroutine write_string_couple(io, i, tab)
C     Write a string couple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i
      
C     Local variables.
      character sbr*19, sdatum(2)*MAX_STRING_LENGTH
      integer n, lens(2)
      
      sbr = 'write_string_couple'
      
      n = 2
      lens(1) = MAX_STRING_LENGTH
      lens(2) = MAX_STRING_LENGTH
      
      call rand_fetch_string_compound_value
     +     (sbr, io, i, n, lens, sdatum)
      
      write(PIO_REPORT, *) tab,
     +     '(', sdatum(1)(1:lens(1)), ', ', sdatum(2)(1:lens(2)), ')'
      
      return
      end
      
      
      subroutine write_int_couple(io, i, j, k, sz, tab)
C     Write an integer couple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, k, sz
      
C     Local variables.
      character sbr*16
      integer idatum1, idatum2
      
C     External function.
      integer rand_fetch_int_value
      
      sbr = 'write_int_couple'
      
      idatum1 = rand_fetch_int_value(sbr, io, i, j, k, sz, 1, 2)
      idatum2 = rand_fetch_int_value(sbr, io, i, j, k, sz, 2, 2)
      
      write(PIO_REPORT, *) tab, '(', idatum1, ', ', idatum2, ')'
      
      return
      end
      
      
      subroutine write_float_couple(io, i, j, k, sz, tab)
C     Write a float couple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, k, sz
      
C     Local variables.
      character sbr*18
      FLOAT fdatum1, fdatum2
      
C     External function.
      FLOAT rand_fetch_float_value
      
      sbr = 'write_float_couple'
      
      fdatum1 = rand_fetch_float_value(sbr, io, i, j, k, sz, 1, 2)
      fdatum2 = rand_fetch_float_value(sbr, io, i, j, k, sz, 2, 2)
      
      write(PIO_REPORT, 1001) tab, fdatum1, fdatum2
      
 1001 format (1x, a,
     +     '(', PRINT_FLOAT_FORMAT, ', ', PRINT_FLOAT_FORMAT, ')')

      return
      end
      
      
      subroutine write_num_couple(io, i, j, k, sz, typ, tab)
C     Write a numeric couple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, k, sz, typ
      
C     External function.
      logical is_scalar_simple_int_type
      
      if (is_scalar_simple_int_type(typ)) then
         call write_int_couple(io, i, j, k, sz, tab)
      else
         call write_float_couple(io, i, j, k, sz, tab)
      endif
      
      return
      end
      
      
      subroutine write_couple(io, i, j, k, sz, typ, tab)
C     Write a couple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, k, sz, typ
      
C     Local variable.
      integer item_typ
      
C     External function.
      logical is_scalar_simple_string_type
      
      item_typ = MAKE_COUPLE_ITEM_TYPE(typ)
      
      if (is_scalar_simple_string_type(item_typ)) then
         call write_string_couple(io, i, tab)
      else
         call write_num_couple(io, i, j, k, sz, item_typ, tab)
      endif
      
      return
      end
      
      
      subroutine write_string_triple(io, i, tab)
C     Write a string triple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i
      
C     Local variables.
      character sbr*19, sdatum(3)*MAX_STRING_LENGTH
      integer n, lens(3)
      
      sbr = 'write_string_triple'
      
      n = 3
      lens(1) = MAX_STRING_LENGTH
      lens(2) = MAX_STRING_LENGTH
      lens(3) = MAX_STRING_LENGTH
      
      call rand_fetch_string_compound_value
     +     (sbr, io, i, n, lens, sdatum)
      
      write(PIO_REPORT, *) tab,
     +     '(', '"', sdatum(1)(1:lens(1)), '"', ', ',
     +     '"', sdatum(2)(1:lens(2)), '"', ', ',
     +     '"', sdatum(3)(1:lens(3)), '"', ')'
      
      return
      end
      
      
      subroutine write_int_triple(io, i, j, k, sz, tab)
C     Write an integer triple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, k, sz
      
C     Local variables.
      character sbr*16
      integer idatum1, idatum2, idatum3
      
C     External function.
      integer rand_fetch_int_value
      
      sbr = 'write_int_triple'
      
      idatum1 = rand_fetch_int_value(sbr, io, i, j, k, sz, 1, 3)
      idatum2 = rand_fetch_int_value(sbr, io, i, j, k, sz, 2, 3)
      idatum3 = rand_fetch_int_value(sbr, io, i, j, k, sz, 3, 3)
      
      write(PIO_REPORT, *) tab,
     +     '(', idatum1, ', ', idatum2, ', ', idatum3, ')'
      
      return
      end
      
      
      subroutine write_float_triple(io, i, j, k, sz, tab)
C     Write a float triple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, k, sz
      
C     Local variables.
      character sbr*18
      FLOAT fdatum1, fdatum2, fdatum3
      
C     External function.
      FLOAT rand_fetch_float_value
      
      sbr = 'write_float_triple'
      
      fdatum1 = rand_fetch_float_value(sbr, io, i, j, k, sz, 1, 3)
      fdatum2 = rand_fetch_float_value(sbr, io, i, j, k, sz, 2, 3)
      fdatum3 = rand_fetch_float_value(sbr, io, i, j, k, sz, 3, 3)
      
      write(PIO_REPORT, 1001) tab, fdatum1, fdatum2, fdatum3
      
 1001 format (1x, a,
     +     '(', PRINT_FLOAT_FORMAT, ', ',
     +     PRINT_FLOAT_FORMAT, ', ',
     +     PRINT_FLOAT_FORMAT, ')')

      return
      end
      
      
      subroutine write_num_triple(io, i, j, k, sz, typ, tab)
C     Write a numeric triple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, k, sz, typ
      
C     External function.
      logical is_scalar_simple_int_type
      
      if (is_scalar_simple_int_type(typ)) then
         call write_int_triple(io, i, j, k, sz, tab)
      else
         call write_float_triple(io, i, j, k, sz, tab)
      endif
      
      return
      end
      
      
      subroutine write_triple(io, i, j, k, sz, typ, tab)
C     Write a triple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, k, sz, typ
      
C     Local variable.
      integer item_typ
      
C     External function.
      logical is_scalar_simple_string_type
      
      item_typ = MAKE_TRIPLE_ITEM_TYPE(typ)
      
      if (is_scalar_simple_string_type(item_typ)) then
         call write_string_triple(io, i, tab)
      else
         call write_num_triple(io, i, j, k, sz, item_typ, tab)
      endif
      
      return
      end
      
      
      subroutine write_tuple(io, i, j, k, sz, typ, tab)
C     Write a tuple value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, k, sz, typ
      
C     External function.
      logical is_scalar_couple_type
      
      if (is_scalar_couple_type(typ)) then
         call write_couple(io, i, j, k, sz, typ, tab)
      else
         call write_triple(io, i, j, k, sz, typ, tab)
      endif
      
      return
      end
      
      
      subroutine write_scalar(io, i, j, k, sz, typ, tab)
C     Write a scalar value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, k, sz, typ
      
C     External function.
      logical is_scalar_simple_type
      
      if (is_scalar_simple_type(typ)) then
         call write_simple(io, i, j, k, sz, typ, tab)
      else
         call write_tuple(io, i, j, k, sz, typ, tab)
      endif
      
      return
      end
      
      
      subroutine write_scalars(io, i, j, typ, sz, tab)
C     Write scalar items of an array value.
      
      implicit none
      
C     Input arguments.
      character tab*(*)
      integer io, i, j, typ, sz
      
C     Local variable.
      integer k
      
      do k = 1, sz
         call write_scalar(io, i, j, k, sz, typ, tab)
      enddo
      
      return
      end
      
      
      subroutine write_vector(io, i, typ)
C     Write a vector value.
      
      implicit none
      
C     Input arguments.
      integer io, i, typ
      
C     Local variables.
      character sbr*12
      integer item_typ, pv, sz
      
C     External function
      integer fetch_value_dimension
      
      sbr = 'write_vector'
      
      item_typ = MAKE_VECTOR_ITEM_TYPE(typ)
      
      pv = fetch_value_dimension(sbr, io, i)
      if (pv .ne. 1) then
         call error_wrong_dimension(sbr, 1, pv)
      endif
      call fetch_value_sizes(sbr, io, i, sz)
      
      call write_scalars(io, i, 1, item_typ, sz, '      ')
      
      return
      end
      
      
      subroutine write_row(io, i, j, typ, sz)
C     Write jth row of a multi-dimensional value.
      
      implicit none
      
C     Input arguments.
      integer io, i, j, typ, sz
      
      write(PIO_REPORT, *) '      [|'
      call write_scalars(io, i, j, typ, sz, '        ')
      write(PIO_REPORT, *) '      |]'
      
      return
      end
      
      
      subroutine write_rows(io, i, typ, pv, sizev, layoutv)
C     Write rows of a multi-dimensional value.
      
      implicit none
      
C     Input arguments.
      integer io, i, typ, pv, sizev(*), layoutv(*)
      
C     Local variables.
      character sbr*10
      integer nb_row, sz, j
      
      sbr = 'write_rows'
      
      call row_number_and_size
     +     (sbr, 'written', pv, sizev, layoutv, nb_row, sz)
      
      do j = 1, nb_row
         call write_row(io, i, j, typ, sz)
      enddo
      
      return
      end
      
      
      subroutine write_multi_d(io, i, typ)
C     Write a multi-dimensional value.
      
      implicit none
      
C     Input arguments.
      integer io, i, typ
      
C     Local variables.
      character sbr*26
      integer item_typ, pv, sizev(MAX_ARRAY_DIMENSION)
      integer lkv, layoutv(MAX_ARRAY_DIMENSION)
      
C     External functions.
      integer fetch_value_dimension
      
      sbr = 'write_multi_d_simple_value'
      
      item_typ = MAKE_MULTI_D_ITEM_TYPE(typ)
      
      pv = fetch_value_dimension(sbr, io, i)
      call fetch_value_sizes(sbr, io, i, sizev)
      call fetch_value_layout(sbr, io, i, lkv, layoutv)
      
      call write_rows(io, i, item_typ, pv, sizev, layoutv)
      
      return
      end
      
      
      subroutine write_array(io, i, typ)
C     Write ith value.
      
      implicit none
      
C     Input argument.
      integer io, i, typ
      
C     External function.
      logical is_vector_type
      
      write(PIO_REPORT, *) '    [|'
      
      if (is_vector_type(typ)) then
         call write_vector(io, i, typ)
      else
         call write_multi_d(io, i, typ)
      endif
      
      write(PIO_REPORT, *) '    |]'
      
      return
      end
      
      
      subroutine write_value(io, i)
C     Write ith value.
      
      implicit none
      
C     Input argument.
      integer io, i
      
C     Local variables.
      character sbr*11
      integer tv, bv, ev
      
C     External functions.
      logical is_scalar_type
      integer fetch_value_type, fetch_value_begin, fetch_value_end
      
      sbr = 'write_value'
      
      tv = fetch_value_type(sbr, io, i)
      
      bv = fetch_value_begin(sbr, io, i)
      ev = fetch_value_end(sbr, io, i)
      
      write(PIO_REPORT, *)
     +     '[Pio/Fortran]   Value [', bv, '; ', ev, '[ ='
      
      if (is_scalar_type(tv)) then
         call write_scalar(io, i, 1, 1, 1, tv, '    ')
      else
         call write_array(io, i, tv)
      endif
      
      return
      end
      
      
      subroutine write_values(io)
C     Write all the values.
      
      implicit none
      
C     Input argument.
      integer io
      
C     Local variables.
      character sbr*12
      integer nv, i
      
C     External function.
      integer fetch_communication_values_number
      
      sbr = 'write_values'
      
      nv = fetch_communication_values_number(sbr, io)
      
      do i = 1, nv
         write(PIO_REPORT, *) ' '
         write(PIO_REPORT, *) '[Pio/Fortran]   Value number ', i
         call write_value(io, i)
      enddo
      
      return
      end
      
      
      subroutine pio_write_memory(io)
C     Write the contents of the memory to stderr.
      
      implicit none
      
      include 'memory_name_definition.fh'
      
C     Input argument.
      integer io
      
      write(PIO_REPORT, *) '[Pio/Fortran] ',
     +     memory_name(io+2), ' memory begins'
      
      call write_communication_name(io)
      call write_values(io)
      
      write(PIO_REPORT, *) ' '
      write(PIO_REPORT, *) '[Pio/Fortran] ',
     +     memory_name(io+2), ' memory ends.'
      
      return
      end
